<?php

namespace Drupal\viewsreference\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;


/**
 *
 * @FieldFormatter(
 *   id = "viewsreference_formatter",
 *   label = @Translation("Views Reference"),
 *   field_types = {"viewsreference"}
 * )
 */
class ViewsReferenceFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $options = parent::defaultSettings();

    $options['render_view'] = TRUE;
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    // We may decide on alternatives to rendering the view so get settings established
    $form['render_view'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Render View'),
      '#default_value' => $this->getSetting('render_view'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();
    $settings = $this->getSettings();

    $summary[] = t('Render View: @view', array('@view' => $settings['render_view'] ? 'TRUE' : 'FALSE'));
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      $view_name = $item->getValue()['target_id'];
      $display_id = $item->getValue()['display_id'];
      $argument = $item->getValue()['argument'];
      $title = $item->getValue()['title'];
      $viewObj = \Drupal\views\Views::getView($view_name);
      // Someone may have deleted the View
      if (!is_object($viewObj)) {
        continue;
      }
      $viewObj->setDisplay($display_id);

      if ($argument != '') {
        $viewObj->setArguments(array($argument));
      }

      if ($title) {
        $title = $viewObj->getTitle();
        $title_render_array = [
          '#markup' => t('@title', ['@title'=> $title]),
          '#allowed_tags' => ['h2'],
          '#attributes' => array(
            'class' => array('view-title')
          )
        ];
      }
      $viewObj->build($display_id);
      $viewObj->execute($display_id);
      $result = $viewObj->result;
      $render = $viewObj->render();
      if ($this->getSetting('render_view')) {
        if ($title && !empty($result)) {
          $elements[$delta]['title'] = $title_render_array;
        }
        $elements[$delta]['contents'] = $render;
      }

    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
//  public static function isApplicable(FieldDefinitionInterface $field_definition) {
//    return $field_definition->getTargetEntityTypeId() === 'user' && $field_definition->getName() === 'name';
//  }

}
